pro msopta_refatm, atmdisk, atmfile, refdisk, reffile, system=system0, composition=composition
;------------------------------------------------------------------------------
;+
; NAME:
;   msopta_refatm ('Matthias_Steffen_opacity_tables_reference_atmosphere')
;
; PURPOSE:
;   Read ATLAS, MARCS or PHOENIX raw data and write it in MSopta reference atmosphere format.
;
; CATEGORY:
;   I/O
;
; CALLING SEQUENCE:
;
; INPUT:
;
; OUTPUT:
;   None
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   rdatlas:
;   rdatlasfmo
;   rdmarcs:
;   rdmarcs2:
;   rdphoenix:
;   rdphoenix20:
;
; SIDE EFFECTS:
;   Write a file.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2007-10-10 Version from HGL
;   2007-10-11 (B.F. Lyon) Phoenix input possible, keyword "system" added
;   2008-02-21 (HGL Meudon) ATLAS FMO format added as another "system"
;   2010-11-25 (HGL Heidelberg) Phoenix .20 format added as another "system"
;-
;------------------------------------------------------------------------------
;
; --- Keyword handling ---
if (keyword_set(system0)) then begin
  system=system0
endif else begin
  system='MARCS'
endelse
;
; Routine for writing reference atmosphere file for msopta
;
if (system eq 'ATLASFMO') then begin 
  compile_routine, 'nngasgas', /no_recompile
  print, 'Reading ', atmdisk+atmfile 
  rdatlasfmo, atmdisk+atmfile, atm, Teff=Teff, grav=grav 
  
  ;       r(*, 0) - RHOX
;       r(*, 1) - TEMP
;       r(*, 2) - PRESSURE
;       r(*, 3) - ELECTRON NUMBER DENSITY
;       r(*, 4) - ROSSELAND MEAN OPACITY
;       r(*, 5) - RADIATIVE ACCELERATION
;       r(*, 6) - TURBULENT VELOCITY
;       r(*, 7) - CONVECTIVE FLUX
;       r(*, 8) - CONVECTIVE VELOCITY
;       r(*, 9) - SOUND VELOCITY 

   print, 'Implementation ATLASFMO incomplete!!'
   return
endif
;
if (system eq 'ATLAS') then begin 
  print, 'Reading ', atmdisk+atmfile 
  rdatlas, atmdisk+atmfile, atm, Teff=Teff, grav=grav 
;
  T  = reverse(reform(atm(*, 1))) 
  P  = reverse(reform(atm(*, 2))) 
  X  = reverse(reform(atm(*, 5))) 
  Q  = reverse(reform(atm(*, 7))) 
  rho= reverse(reform(atm(*, 4))) 
  S  = reverse(reform(atm(*, 0))) 
  Z  = reverse(reform(atm(*,12))) 
  Z  = 1.D5*Z                     
endif
;
if (system eq 'MARCS') then begin 
  print, 'Reading ', atmdisk+atmfile 
  ndot     = strpos(atmfile, '.', /reverse_search) 
  next     = strlen(atmfile)-ndot-1 
  extension=strmid(atmfile,ndot+1,next) 
  ; Use rdmarcs for .mod atmosphere files
  if (extension eq 'mod') then begin 
     atm=0.0 & Teff=0.0 & grav=0.0 
     rdmarcs, atmdisk+atmfile, atm, Teff=Teff, grav=grav 
  endif    
  ; Use rdmarcs2 for .wallpaper atmosphere files
  if (extension eq 'wallpaper') then begin 
     jstop = 56 
     rdmarcs2, atmdisk+atmfile, atm, jstop, Teff=Teff, grav=grav 
  endif  
  ;
  T=   reform(atm(4, *)) 
  P=   reform(atm(6, *)) 
  X=   reform(atm(11,*)) 
  Q=10^reform(atm(1, *)) 
  rho= reform(atm(12,*)) 
  S=   reform(atm(16,*)) 
  Z=  -reform(atm(3, *)) 
endif
;
if (strupcase(system) eq 'PHOENIX') then begin 
  print, 'Reading ', atmdisk+atmfile 
  rdphoenix, atmdisk+atmfile, atm, Teff=Teff, grav=grav 
  ;
  T=   reform(atm( 2,*)) 
  P=   reform(atm( 3,*)) 
  X=   reform(atm(10,*)) 
  Q=   reform(atm(12,*)) 
  rho= reform(atm( 5,*)) 
  S=   reform(atm(11,*)) 
  Z=   reform(atm(13,*)) 
endif

if (strupcase(system) eq 'PHOENIX20') then begin 
  print, 'Reading ', atmdisk+atmfile 
  rdphoenix20, atmdisk+atmfile, atm, Teff=Teff, grav=grav 
  ;
  T=   reform(atm( 1,*)) 
  P=   reform(atm( 8,*)) 
  X=   reform(atm( 7,*)) 
  rho= reform(atm(10,*)) 
  Z=   reform(atm(16,*)) 
  S=   p/grav        ; rhox, assuming negligible turbulent pressure
  Q=   monocubic(x/grav,p,/integral)+x[0]*p[0]/grav ; tau_ross, assuming assuming negligible turbulent pressure
endif
;
nstop = n_elements(T)
;
get_lun, unit
openw, unit, refdisk+reffile
printf, unit, 'Reference Atmosphere for msopta tau sorting'
printf, unit, 'Extracted from ' + atmfile
printf, unit, '   Teff           grav'
printf, unit, Teff, grav, format='(2E15.6)'
printf, unit
printf, unit, '   Temp           Pgas           Kappa_Ross     Tau_Ross ' + $
              '      rho            rhox           Z           '
printf, unit, '=========================================================' + $
              '================================================'
printf, unit, nstop, format='(I4)'
for n=0,nstop-1 do begin 
  printf, unit, T(n), P(n), X(n), Q(n), rho(n), S(n), Z(n), $
          format='(7E15.6)' 
endfor
;
close, unit
free_lun, unit

end
